@@ -16,7 +16,7 @@ package Text::CSV_XS;
 # Based on Text::CSV by:
 #    Alan Citterman <alan@mfgrtl.com>
 #
-# Extended by:
+# Extended and Remodelled by:
 #    H.Merijn Brand (h.m.brand@xs4all.nl)
 #
 ############################################################################
@@ -30,8 +30,9 @@ use DynaLoader ();
 use Carp;
 
 use vars   qw( $VERSION @ISA );
-$VERSION = "0.52";
+$VERSION = "0.58";
 @ISA     = qw( DynaLoader );
+bootstrap Text::CSV_XS $VERSION;
 
 sub PV { 0 }
 sub IV { 1 }
@@ -67,7 +68,6 @@ my %def_attr = (
     verbatim		=> 0,
     types		=> undef,
 
-
     _EOF		=> 0,
     _STATUS		=> undef,
     _FIELDS		=> undef,
@@ -77,7 +77,7 @@ my %def_attr = (
     _COLUMN_NAMES	=> undef,
     _BOUND_COLUMNS	=> undef,
     );
-my $last_new_err = "";
+my $last_new_err = Text::CSV_XS->SetDiag (0);
 
 sub new
 {
@@ -92,8 +92,9 @@ sub new
 	$last_new_err = "Unknown attribute '$_'";
 	return;
 	}
-    $last_new_err = "";
+    $last_new_err = SetDiag (undef, 0);
     my $self  = {%def_attr, %{$attr}};
+    defined $\ && !exists $attr->{eol} and $self->{eol} = $\;
     bless $self, $class;
     defined $self->{types} and $self->types ($self->{types});
     $self;
@@ -124,10 +125,11 @@ my %_cache_id = (	# Keep in sync with XS!
 sub _set_attr_C
 {
     my ($self, $name, $val) = @_;
+    defined $val or $val = 0;
     $self->{$name} = $val;
     $self->{_CACHE} or return;
     my @cache = unpack "C*", $self->{_CACHE};
-    $cache[$_cache_id{$name}] = defined $val ? unpack "C", $val : 0;
+    $cache[$_cache_id{$name}] = unpack "C", $val;
     $self->{_CACHE} = pack "C*", @cache;
     } # _set_attr_C
 
@@ -135,10 +137,9 @@ sub _set_attr_N
 {
     my ($self, $name, $val) = @_;
     $self->{$name} = $val;
-    $self->{_CACHE} or return;
     my @cache = unpack "C*", $self->{_CACHE};
     my $i = $_cache_id{$name};
-    $cache[$i++] = $_ for unpack "C*", pack "N", defined $val ? $val : 0;
+    $cache[$i++] = $_ for unpack "C*", pack "N", $val;
     $self->{_CACHE} = pack "C*", @cache;
     } # _set_attr_N
 
@@ -170,6 +171,7 @@ sub eol
     my $self = shift;
     if (@_) {
 	my $eol = shift;
+	defined $eol or $eol = "";
 	my $eol_len = length $eol;
 	$self->{eol} = $eol;
 	$self->{_CACHE} or return;
@@ -456,8 +458,6 @@ sub getline_hr
     \%hr;
     } # getline_hr
 
-bootstrap Text::CSV_XS $VERSION;
-
 sub types
 {
     my $self = shift;
@@ -528,7 +528,7 @@ perhaps better called ASV (anything separated values) rather than just CSV.
 =head2 Embedded newlines
 
 B<Important Note>: The default behavior is to only accept ascii characters.
-This means that fields can not contain newlines. If your data contains 
+This means that fields can not contain newlines. If your data contains
 newlines embedded in fields, or characters above 0x7e (tilde), or binary data,
 you *must* set C<< binary => 1 >> in the call to C<new ()>.  To cover the widest
 range of parsing options, you will always want to set binary.
@@ -556,7 +556,7 @@ or, more safely in perl 5.6 and up
  open my $io, "<", $file or die "$file: $!";
  while (my $row = $csv->getline ($io)) {
      my @fields = @$row;
- 
+
 =head2 Unicode (UTF8)
 
 On parsing (both for C<getline ()> and C<parse ()>), if the source is
@@ -566,6 +566,22 @@ are marked binary will also be marked UTF8.
 On combining (C<print ()> and C<combine ()>), if any of the combining
 fields was marked UTF8, the resulting string will be marked UTF8.
 
+For complete control over encoding, please use Text::CSV::Encoded:
+
+    use Text::CSV::Encoded;
+    my $csv = Text::CSV::Encoded->new ({
+        encoding_in  => "iso-8859-1", # the encoding comes into   Perl
+        encoding_out => "cp1252",     # the encoding comes out of Perl
+        });
+
+    $csv = Text::CSV::Encoded->new ({ encoding  => "utf8" });
+    # combine () and print () accept *literally* utf8 encoded data
+    # parse () and getline () return *literally* utf8 encoded data
+
+    $csv = Text::CSV::Encoded->new ({ encoding  => undef }); # default
+    # combine () and print () accept UTF8 marked data
+    # parse () and getline () return UTF8 marked data
+
 =head1 SPECIFICATION
 
 While no formal specification for CSV exists, RFC 4180 1) describes a common
@@ -664,9 +680,10 @@ Currently the following attributes are available:
 
 =item eol
 
-An end-of-line string to add to rows, usually C<undef> (nothing,
-default), C<"\012"> (Line Feed) or C<"\015\012"> (Carriage Return,
-Line Feed). Cannot be longer than 7 (ASCII) characters.
+An end-of-line string to add to rows. C<undef> is replaced with an
+empty string. The default is C<$\>. Common values for C<eol> are
+C<"\012"> (Line Feed) or C<"\015\012"> (Carriage Return, Line Feed).
+Cannot be longer than 7 (ASCII) characters.
 
 If both C<$/> and C<eol> equal C<"\015">, parsing lines that end on
 only a Carriage Return without Line Feed, will be C<parse>d correct.
@@ -682,6 +699,8 @@ Limited to a single-byte character, usually in the range from 0x20
 The separation character can not be equal to the quote character.
 The separation character can not be equal to the escape character.
 
+See also CAVEATS
+
 =item allow_whitespace
 
 When this option is set to true, whitespace (TAB's and SPACE's)
@@ -769,7 +788,7 @@ doubling the quote mark in a field escapes it:
   "foo","bar","Escape ""quote mark"" with two ""quote marks""","baz"
 
 If you change the default quote_char without changing the default
-escape_char, the escape_char will still be the quote mark.  If instead 
+escape_char, the escape_char will still be the quote mark.  If instead
 you want to escape the quote_char by doubling it, you will need to change
 the escape_char to be the same as what you changed the quote_char to.
 
@@ -867,7 +886,7 @@ is equivalent to
      quote_char          => '"',
      escape_char         => '"',
      sep_char            => ',',
-     eol                 => '',
+     eol                 => $\,
      always_quote        => 0,
      binary              => 0,
      keep_meta_info      => 0,
@@ -972,7 +991,7 @@ methods are meaningless, again.
 
 The C<getline_hr ()> and C<column_names ()> methods work together to allow
 you to have rows returned as hashrefs. You must call C<column_names ()>
-first to declare your column names. 
+first to declare your column names.
 
  $csv->column_names (qw( code name price description ));
  $hr = $csv->getline_hr ($io);
@@ -1182,25 +1201,6 @@ as the API may change in future releases.
 
 =head1 EXAMPLES
 
-An example for creating CSV files:
-
-  my $csv = Text::CSV_XS->new;
-
-  open my $csv_fh, ">", "hello.csv" or die "hello.csv: $!";
-
-  my @sample_input_fields = (
-      'You said, "Hello!"',   5.67,
-      '"Surely"',   '',   '3.14159');
-  if ($csv->combine (@sample_input_fields)) {
-      my $string = $csv->string;
-      print $csv_fh "$string\n";
-      }
-  else {
-      my $err = $csv->error_input;
-      print "combine () failed on argument: ", $err, "\n";
-      }
-  close $csv_fh or die "hello.csv: $!";
-
 An example for parsing CSV strings:
 
   my $csv = Text::CSV_XS->new ({ keep_meta_info => 1, binary => 1 });
@@ -1220,7 +1220,27 @@ An example for parsing CSV strings:
       $csv->error_diag ();
       }
 
-Dumping the content of a database ($dbh) table ($tbl) to CSV:
+An example for creating CSV files:
+
+  my $csv = Text::CSV_XS->new;
+
+  open my $csv_fh, ">", "hello.csv" or die "hello.csv: $!";
+
+  my @sample_input_fields = (
+      'You said, "Hello!"',   5.67,
+      '"Surely"',   '',   '3.14159');
+  if ($csv->combine (@sample_input_fields)) {
+      my $string = $csv->string;
+      print $csv_fh "$string\n";
+      }
+  else {
+      my $err = $csv->error_input;
+      print "combine () failed on argument: ", $err, "\n";
+      }
+  close $csv_fh or die "hello.csv: $!";
+
+Or using the C<print ()> method, which is faster like in
+dumping the content of a database ($dbh) table ($tbl) to CSV:
 
   my $csv = Text::CSV_XS->new ({ binary => 1, eol => $/ });
   open my $fh, ">", "$tbl.csv" or die "$tbl.csv: $!";
@@ -1239,14 +1259,32 @@ Reading a CSV file line by line:
   while (my $row = $csv->getline ($fh)) {
       # do something with @$row
       }
-  close $fh or die "file.csv: $!";;
+  $csv->eof or $csv->error_diag;
+  close $fh or die "file.csv: $!";
 
 For more extended examples, see the C<examples/> subdirectory in the
 original distribution. Included is C<examples/parser-xs.pl>, that could
-be used to `fix' bad CSV
+be used to `fix' bad CSV and parse beyond errors.
 
   perl examples/parser-xs.pl bad.csv >good.csv
 
+=head1 CAVEATS
+
+C<Text::CSV_XS> is not designed to detect the characters used for field
+separation and quoting. The parsing is done using predefined settings. In
+the examples subdirectory, you can find scripts that demonstrate how you
+can try to detect these characters yourself.
+
+=head2 Microsoft Excel
+
+The import/export from Microsoft Excel is a I<risky task>, according to the
+documentation in C<Text::CSV::Separator>. Microsoft uses the system's default
+list separator defined in the regional settings, which happens to be a
+semicolon for Dutch, German and Spanish (and probably some others as well).
+For the English locale, the default is a comma. In Windows however, the user
+is free to choose a predefined locale, and then change every individual
+setting in it, so checking the locale is no solution.
+
 =head1 TODO
 
 =over 2
@@ -1267,14 +1305,6 @@ error_diag is a (very) good start, but there is more work to be done here.
 Basic calls should croak or warn on illegal parameters. Errors should be
 documented.
 
-=item eol
-
-Discuss an option to make the eol honor the $/ setting. Maybe
-
-  my $csv = Text::CSV_XS->new ({ eol => $/ });
-
-is already enough, and new options only make things less opaque.
-
 =item setting meta info
 
 Future extensions might include extending the C<meta_info ()>,
@@ -1297,17 +1327,6 @@ Using C<getline ()> and C<print ()> instead is the prefered way to go.
 We probably need many more tests to check if all edge-cases are covered.
 See t/50_utf8.t.
 
-Probably the best way to do this more reliable is to make a subclass
-Text::CSV_XS::Encoded that can be passed the required encoding and
-then behaves transparently (but slower), something like this:
-
-    use Text::CSV::Encoded;
-    my $csv = Text::CSV::Encoded->new ({
-        encoding     => "utf-8",      # Both in and out
-        encoding_in  => "iso-8859-1", # Only the input
-        encoding_out => "cp1252",     # Only the output
-        });
-
 =item Parse the whole file at once
 
 Implement a new methods that enables the parsing of a complete file
@@ -1350,11 +1369,6 @@ No guarantees, but this is what I have in mind right now:
 
 =item next + 1
 
- - allow_double_quoted
- - Text::CSV_XS::Encoded (maybe)
-
-=item next + 2
-
  - csv2csv - a script to regenerate a CSV file to follow standards
  - EBCDIC support
 
@@ -1441,19 +1455,34 @@ has been selected with the constructor.
 Sequences like C<1,"foo\rbar",2> are only allowed when the binary option
 has been selected with the constructor.
 
-=item 2023 "EIQ - QUO ..."
+=item 2023 "EIQ - QUO character not allowed
 
 Sequences like C<"foo "bar" baz",quux> and C<2023,",2008-04-05,"Foo, Bar",\n>
 will cause this error.
 
 =item 2024 "EIQ - EOF cannot be escaped, not even inside quotes"
 
+The escape character is not allowed as last character in an input stream.
+
 =item 2025 "EIQ - Loose unescaped escape"
 
+An escape character should escape only characters that need escaping. Allowing
+the escape for other characters is possible with the C<allow_loose_escape>
+attribute.
+
 =item 2026 "EIQ - Binary character inside quoted field, binary off"
 
+Binary characters are not allowed by default. Exceptions are fields that
+contain valid UTF-8, that will automatically be upgraded is the content is
+valid UTF-8. Pass the C<binary> attribute with a true value to accept binary
+characters.
+
 =item 2027 "EIQ - Quoted field not terminated"
 
+When parsing a field that started with a quotation character, the field is
+expected to be closed with a quotation character. When the parsed line is
+exhausted before the quote is found, that field is not terminated.
+
 =item 2030 "EIF - NL char inside unquoted verbatim, binary off"
 
 =item 2031 "EIF - CR char is first char of field, not part of EOL"
@@ -1470,6 +1499,8 @@ will cause this error.
 
 =item 2110 "ECB - Binary character in Combine, binary off"
 
+=item 2200 "EIO - print to IO failed. See errno"
+
 =item 3001 "EHR - Unsupported syntax for column_names ()"
 
 =item 3002 "EHR - getline_hr () called before column_names ()"
@@ -1488,8 +1519,9 @@ will cause this error.
 
 =head1 SEE ALSO
 
-L<perl(1)>, L<IO::File(3)>, L{IO::Handle(3)>, L<IO::Wrap(3)>,
-L<Text::CSV(3)>, L<Text::CSV_PP(3)>.  and L<Spreadsheet::Read(3)>.
+L<perl(1)>, L<IO::File(3)>, L<IO::Handle(3)>, L<IO::Wrap(3)>,
+L<Text::CSV(3)>, L<Text::CSV_PP(3)>, L<Text::CSV::Encoded>,
+L<Text::CSV::Separator(3)>, and L<Spreadsheet::Read(3)>.
 
 =head1 AUTHORS and MAINTAINERS
 
